import unittest
#import Zope

from Testing import ZopeTestCase
ZopeTestCase.installProduct('SilvaReferenceChecker')

from Products.Silva.tests.SilvaTestCase import SilvaTestCase

from DateTime import DateTime
from Products.ParsedXML.ParsedXML import ParsedXML
from Products.Silva.Ghost import GhostVersion

import Products.SilvaReferenceChecker.ReferenceChecker
from os.path import dirname, join, split

dir, _  = split(Products.SilvaReferenceChecker.ReferenceChecker.__file__)
testimage = join(dir,'tests/testimage.gif')
README = join(dir,'README.txt')


class ReferenceCheckerTestCase(SilvaTestCase):

    def addFolderWithIndex(self, id, title, **kw):
        return self.add_folder(self.root, id, title,
                               policy_name='Silva Document', **kw )

    def afterSetUp(self):
        # the preinstalled root has an "index" which we dont want to check here
        self.root.get_default().close_version()

        # example (Zope) image ... not yet used
        self.root.manage_addFolder('images',"Images dummy")
        self.root.images.manage_addImage('testimage.gif', open(testimage, 'rb'),
                                          content_type='image/gif')


        self.rc = self.root.manage_addProduct['SilvaReferenceChecker'].manage_addReferenceChecker()
        self.assertEqual(self.rc.meta_type, 'Silva Reference Checker')
        # self.rc._default_abs_root = ReferenceInfo.PATH_SILVA_ROOT

        # Doc, Folder
        self.doc1 = self.add_document(self.root, 'test_doc1', 'Test doc')

        self.folder = self.addFolderWithIndex('test_folder', 'Test Folder')
        self.folder_foo = self.addFolderWithIndex('test_folder_foo',
                                                  'Test Folder for Weird Acquisition')

        self.doc2 = self.add_document(self.folder, 'test_doc2', 'Test doc 2' )
        self.doc3 = self.add_document(self.folder_foo, 'test_doc3', 'Test doc 3')
        self.doc4 = self.add_document(self.folder_foo, 'test_doc4', 'Test doc 4')

        # Image
        self.silva_image1 = self.add_image(self.folder, 'silva_image1','Silva Image1',
                                           file=open(testimage, 'rb'))

        # Ghost 1 correct 3 wrong type
        self.ghost1 = self.add_ghost(self.folder_foo, 'test_ghost1',
                                     '/root/test_folder/test_doc2')
        self.ghost2 = self.add_ghost(self.folder_foo,'test_ghost2',
                                     '/root/test_folder_foo/test_ghost1')
        self.ghost3 = self.add_ghost(self.folder_foo,'test_ghost3',
                                     '/root/test_folder')
        self.ghost4 = self.add_ghost(self.folder_foo,'test_ghost4',
                                     '/root/test_folder/silva_image1')
        self.folder_ghost = self.addObject(self.folder_foo,'GhostFolder',
                                     'test_folderghost',
                                     content_url='/root/test_folder')

        self.silva_file1 = self.addObject(self.folder, 'File', 'silva_file1',
                                          title='Silva File1',
                                          file=open(README, 'rb'))

        # Check test objects
        self.assertEqual(self.root.id, 'root')
        self.assertEqual(self.root.meta_type, 'Silva Root')
        self.assertEqual(self.doc1.getPhysicalPath(), ('', 'root', 'test_doc1'))

        self.assertEqual(self.folder.getPhysicalPath(), ('', 'root', 'test_folder'))
        self.assertEqual(self.doc2.getPhysicalPath(), ('', 'root', 'test_folder', 'test_doc2') )

        self.assertEqual(self.folder_foo.getPhysicalPath(), ('', 'root', 'test_folder_foo'))
        self.assertEqual(self.doc3.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_doc3') )
        self.assertEqual(self.doc4.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_doc4') )

        self.assertEqual(self.silva_image1.getPhysicalPath(), ('', 'root', 'test_folder', 'silva_image1') )

        self.assertEqual(self.ghost1.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost1') )
        self.assertEqual(self.ghost2.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost2') )
        self.assertEqual(self.ghost3.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost3') )
        self.assertEqual(self.ghost4.getPhysicalPath(), ('', 'root', 'test_folder_foo', 'test_ghost4') )

        # Publish folder
        self.folder.get_default().set_unapproved_version_publication_datetime(DateTime()-1)
        self.folder.get_default().approve_version()
        self.folder_foo.get_default().set_unapproved_version_publication_datetime(DateTime()-1)
        self.folder_foo.get_default().approve_version()

        # Fill/publish documents
        self._fillAndPublishDoc1()
        self._fillAndPublishDoc2()
        self._fillAndPublishDoc3()
        # self.doc4 is unpublished!
        self._publish(self.ghost1)
        self._publish(self.ghost2)
        self._publish(self.ghost3)
        self._publish(self.ghost4)


    def _publish(self, doc):
        assertTrue = self.assert_
        assertFalse = self.failIf
        assertFalse(doc.get_viewable())
        doc.set_unapproved_version_publication_datetime(DateTime()-1)
        doc.approve_version()
        assertTrue(doc.get_viewable())
        

    def _fillAndPublishDoc1(self):
        assertTrue = self.assert_
        assertFalse = self.failIf

        # The first three references are ok.
        # Then there is a case of
        # - unpublished reference,
        # - weird acquisition,
        # - wrong type, and
        # - missing
        # Then we have some special cases:
        # - embedding an hires image
        # - link to an hires image (around the same image here)
        # - link to an existing anchor
        # - two links to nonexisting anchors
        # Total: 11
        # Broken: 6
        xml = """\
        <doc>
        <index title="A test index, no reference" name="some_index"/>
        <p>Downloads</p>
        <p><link url="test_folder/test_doc2">Ok</link></p>
        <image link="/test_folder/test_doc2" path="/root/test_folder/silva_image1"></image>
        <p><link url="/test_folder_foo/test_doc4">Unpublished</link></p>
        <p><link url="/test_folder/test_folder_foo/test_doc3">Weird Acquisition</link></p>
        <image path="/root/test_folder/test_doc2"></image>
        <p><link url="/test_folder_foo/test_doc2">Broken</link></p>
        <image target="_blank" title="" link_to_hires="1"
               link="/test_folder/silva_image1?hires" path="/root/test_folder/silva_image1?hires" alignment="" />
        <p><link url="test_doc1#some_index" target="_self" title="">Text</link></p>
        <p><link url="test_doc1#wrong_index" target="_self" title="">Text</link></p>
        <p><link url="test_folder/test_doc2#wrong_index2" target="_self" title="">Text</link></p>
        </doc>
        """

        # XXX things we dont check yet: HiRes Images
        # <image target="_blank" title="" link_to_hires="1" link="image.png?hires" path="image.png?hires" alignment=""/>
        # <link url="image.png?hires" target="_self" title="">Text</link><
        # also Missing:
        #  - links to folders (I think)
        #  - links to external sources ? (--> keep extra)

        getattr(self.doc1,'0').content = ParsedXML('content', unicode(xml, 'latin_1'))
        # publish the document
        self._publish(self.doc1)

    def _fillAndPublishDoc2(self):
        # XXX avoid dependencies on editorsupport: fill in raw xml instead
        assertTrue = self.assert_
        assertFalse = self.failIf
        
        # put some content into paragraph node including links
        # The first two internal references are ok.
        # The first external reference is missing, the second is ok :-)
        # Then there is a missing internal reference and a mailto reference.
        # finally a (broken) "anchor only" reference
        # Total: 7
        # Broken: 3
        xml_raw = """<doc><p>This is a helper text with <link url="test_doc2">a <em>relative</em> link</link>,<br />
        An <link target="_blank" url="/test_doc1">absolute link</link> and<br />
        a <link target="_blank" url="../test_doc1">relative</link> link to the same document.<br />
        A <link target="popup" url="http://foo.bar/test_doc1">broken external link</link>.<br />
        An external link that usually works: <link url="http://www.zlb.de/">zlb</link>.<br />
        A <link url="/to_nowhere/doc883">broken internal link</link>.<br />
        A <link url="mailto:someone@somewhere.net">special link</link> at the end.</p>
        <p><link url="#no_anchor_in_index" target="_self" title="">No anchor in index</link></p>
        </doc>"""
        getattr(self.doc2,'0').content.manage_edit( unicode(xml_raw, 'latin_1') )
        # publish the document
        self._publish(self.doc2)
        #print self.doc2.view()


    def _fillAndPublishDoc3(self):
        assertTrue = self.assert_
        assertFalse = self.failIf
        xml = """\
        <doc>
        <p>Nothing</p>
        </doc>
        """
        getattr(self.doc3,'0').content = ParsedXML('content', unicode(xml, 'latin_1'))
        # publish the document
        self._publish(self.doc3)

    def pr(self, refs):
        # print references: for debugging
        for r in refs: print r
        
    def testAInternalLinkBasics(self):
        assertEquals = self.assertEquals
        assertTrue = self.assert_
        assertFalse = self.failIf

        rc = self.rc
        # collect references
        references = []
        rc._get_references_helper(rc.aq_inner.aq_parent, references)
        # self.pr(references)
        assertEquals(25, len(references))

        rc._check_internal_references()
        references = rc._internal_references._list
        # self.pr(references)

        assertEquals(22, len([r for r in references if r.ref_type==r.REF_INTERN]) )
        assertEquals( 3, len([r for r in references if r.restriction==r.RESTRICT_IMAGE]) )

        assertEquals(3, len([r for r in references if r.status==r.STATUS_WRONG_TYPE]) )
        assertEquals(1, len([r for r in references if r.status==r.STATUS_UNPUBLISHED]) )
        assertEquals(2, len([r for r in references if r.status==r.STATUS_MISSING]) )
        assertEquals(3, len([r for r in references if r.status==r.STATUS_MISSING_ANCHOR]) )
        assertEquals(1, len([r for r in references if r.status==r.STATUS_WEIRD_ACQUISITION]) )
        assertEquals(10, len(rc.get_broken_internal_references()) )
        
    def testExternalLinkBasics(self):
        assertEquals = self.assertEquals
        assertTrue = self.assert_
        assertFalse = self.failIf
        rc = self.rc

        # collect references
        references = []
        rc._get_references_helper(rc.get_root(), references)
        #self.pr(references)

        # determine type
        rc._determine_reference_types(references)
        #self.pr(references)
        assertEquals(len([r for r in references if r.ref_type==r.REF_EXTERN]), 2)
        assertEquals(len([r for r in references if r.ref_type==r.REF_MAILTO]), 1)
        # check
        rc._sleep_after_external_checks = 5.0 # default is 30
        rc._check_external_references()
        assertEquals(len(rc.get_broken_external_references()), 1)
        references = rc._external_references._list
        #self.pr(references)
        

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReferenceCheckerTestCase, 'test'))
    return suite        

def main():
    unittest.TextTestRunner(verbosity=1).run(test_suite())

if __name__ == '__main__':
    main()
